package java.util;

class ComparableTimSort
{
    private static final int MIN_MERGE = 32;
    private final Object[] a;
    private static final int MIN_GALLOP = 7;
    private int minGallop;
    private static final int INITIAL_TMP_STORAGE_LENGTH = 256;
    private Object[] tmp;
    private int tmpBase;
    private int tmpLen;
    private int stackSize;
    private final int[] runBase;
    private final int[] runLen;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    private ComparableTimSort(Object[] a, Object[] tmp, int tmpBase, int tmpLen) {
        this.minGallop = 7;
        this.stackSize = 0;
        this.a = a;
        int length = a.length;
        int tmpLen2 = (length < 512) ? (length >>> 1) : 256;
        if (tmp == null || tmpLen < tmpLen2 || tmpBase + tmpLen2 > tmp.length) {
            this.tmp = new Object[tmpLen2];
            this.tmpBase = 0;
            this.tmpLen = tmpLen2;
        }
        else {
            this.tmp = tmp;
            this.tmpBase = tmpBase;
            this.tmpLen = tmpLen;
        }
        int n = (length < 120) ? 5 : ((length < 1542) ? 10 : ((length < 119151) ? 24 : 49));
        this.runBase = new int[n];
        this.runLen = new int[n];
    }
    
    static void sort(Object[] array, int n, int n2, Object[] array2, int n3, int n4) {
        assert array != null && n >= 0 && n <= n2 && n2 <= array.length;
        int i = n2 - n;
        if (i < 2) {
            return;
        }
        if (i < 32) {
            binarySort(array, n, n2, n + countRunAndMakeAscending(array, n, n2));
            return;
        }
        ComparableTimSort comparableTimSort = new ComparableTimSort(array, array2, n3, n4);
        int minRunLength = minRunLength(i);
        do {
            int countRunAndMakeAscending = countRunAndMakeAscending(array, n, n2);
            if (countRunAndMakeAscending < minRunLength) {
                minRunLength = ((i <= minRunLength) ? i : minRunLength);
                binarySort(array, n, n + minRunLength, n + countRunAndMakeAscending);
                countRunAndMakeAscending = minRunLength;
            }
            comparableTimSort.pushRun(n, countRunAndMakeAscending);
            comparableTimSort.mergeCollapse();
            n += countRunAndMakeAscending;
            i -= countRunAndMakeAscending;
        } while (i != 0);
        assert n == n2;
        comparableTimSort.mergeForceCollapse();
        assert comparableTimSort.stackSize == 1;
        return;
    }
    
    private static void binarySort(Object[] array, int i, int n, int j) {
        assert i <= j && j <= n;
        if (j == i) {
            ++j;
        }
        while (j < n) {
            Comparable<Object> comparable = (Comparable)array[j];
            assert i <= j;
            while (i < j) {
                int n2 = i + j >>> 1;
                if (comparable.compareTo(array[n2]) < 0) {
                    j = n2;
                }
                else {
                    i = n2 + 1;
                }
            }
            assert i == j;
            int n3 = j - i;
            switch (n3) {
                case 2: {
                    array[i + 2] = array[i + 1];
                }
                case 1: {
                    array[i + 1] = array[i];
                    break;
                }
                default: {
                    System.arraycopy(array, i, array, i + 1, n3);
                    break;
                }
            }
            array[i] = comparable;
            ++j;
        }
        return;
    }
    
    private static int countRunAndMakeAscending(Object[] array, int n, int n2) {
        assert n < n2;
        int n3 = n + 1;
        if (n3 == n2) {
            return 1;
        }
        if (((Comparable)array[n3++]).compareTo(array[n]) < 0) {
            while (n3 < n2 && ((Comparable)array[n3]).compareTo(array[n3 - 1]) < 0) {
                ++n3;
            }
            reverseRange(array, n, n3);
        }
        else {
            while (n3 < n2 && ((Comparable)array[n3]).compareTo(array[n3 - 1]) >= 0) {
                ++n3;
            }
        }
        return n3 - n;
    }
    
    private static void reverseRange(Object[] array, int i, int n) {
        --n;
        while (i < n) {
            Object o = array[i];
            array[i++] = array[n];
            array[n--] = o;
        }
    }
    
    private static int minRunLength(int i) {
        assert i >= 0;
        int n = 0;
        while (i >= 32) {
            n |= (i & 0x1);
            i >>= 1;
        }
        return i + n;
    }
    
    private void pushRun(int n, int n2) {
        this.runBase[this.stackSize] = n;
        this.runLen[this.stackSize] = n2;
        ++this.stackSize;
    }
    
    private void mergeCollapse() {
        while (this.stackSize > 1) {
            int n = this.stackSize - 2;
            if (n > 0 && this.runLen[n - 1] <= this.runLen[n] + this.runLen[n + 1]) {
                if (this.runLen[n - 1] < this.runLen[n + 1]) {
                    --n;
                }
                this.mergeAt(n);
            }
            else if (this.runLen[n] <= this.runLen[n + 1]) {
                this.mergeAt(n);
            }
            else {
                break;
            }
        }
    }
    
    private void mergeForceCollapse() {
        while (this.stackSize > 1) {
            int n = this.stackSize - 2;
            if (n > 0 && this.runLen[n - 1] < this.runLen[n + 1]) {
                --n;
            }
            this.mergeAt(n);
        }
    }
    
    private void mergeAt(int n) {
        assert this.stackSize >= 2;
        assert n >= 0;
        assert n == this.stackSize - 3;
        int n2 = this.runBase[n];
        int n3 = this.runLen[n];
        int n4 = this.runBase[n + 1];
        int n5 = this.runLen[n + 1];
        assert n3 > 0 && n5 > 0;
        assert n2 + n3 == n4;
        this.runLen[n] = n3 + n5;
        if (n == this.stackSize - 3) {
            this.runBase[n + 1] = this.runBase[n + 2];
            this.runLen[n + 1] = this.runLen[n + 2];
        }
        --this.stackSize;
        int gallopRight = gallopRight((Comparable)this.a[n4], this.a, n2, n3, 0);
        assert gallopRight >= 0;
        int n6 = n2 + gallopRight;
        int n7 = n3 - gallopRight;
        if (n7 == 0) {
            return;
        }
        int gallopLeft = gallopLeft((Comparable)this.a[n6 + n7 - 1], this.a, n4, n5, n5 - 1);
        assert gallopLeft >= 0;
        if (gallopLeft == 0) {
            return;
        }
        if (n7 <= gallopLeft) {
            this.mergeLo(n6, n7, n4, gallopLeft);
        }
        else {
            this.mergeHi(n6, n7, n4, gallopLeft);
        }
        return;
    }
    
    private static int gallopLeft(Comparable<Object> comparable, Object[] array, int n, int n2, int n3) {
        assert n2 > 0 && n3 >= 0 && n3 < n2;
        int n4 = 0;
        int n5 = 1;
        int i;
        int n7;
        if (comparable.compareTo(array[n + n3]) > 0) {
            int n8 = n2 - n3;
            while (n5 < n8 && comparable.compareTo(array[n + n3 + n5]) > 0) {
                n4 = n5;
                n5 = (n5 << 1) + 1;
                if (n5 > 0) {
                    continue;
                }
                n5 = n8;
            }
            if (n5 > n8) {
                n5 = n8;
            }
            i = n4 + n3;
            n7 = n5 + n3;
        }
        else {
            int n6 = n3 + 1;
            while (n5 < n6 && comparable.compareTo(array[n + n3 - n5]) <= 0) {
                n4 = n5;
                n5 = (n5 << 1) + 1;
                if (n5 > 0) {
                    continue;
                }
                n5 = n6;
            }
            if (n5 > n6) {
                n5 = n6;
            }
            i = n3 - n5;
            n7 = n3 - n4;
        }
        assert -1 <= i && i < n7 && n7 <= n2;
        ++i;
        while (i < n7) {
            int n9 = i + (n7 - i >>> 1);
            if (comparable.compareTo(array[n + n9]) > 0) {
                i = n9 + 1;
            }
            else {
                n7 = n9;
            }
        }
        assert i == n7;
        return n7;
    }
    
    private static int gallopRight(Comparable<Object> comparable, Object[] array, int n, int n2, int n3) {
        assert n2 > 0 && n3 >= 0 && n3 < n2;
        int n4 = 1;
        n4 = 0;
        int i;
        int n6;
        if (comparable.compareTo(array[n + n3]) < 0) {
            int n7 = n3 + 1;
            while (n4 < n7 && comparable.compareTo(array[n + n3 - n4]) < 0) {
                n4 = (n4 << 1) + 1;
                if (n4 > 0) {
                    continue;
                }
                n4 = n7;
            }
            if (n4 > n7) {
                n4 = n7;
            }
            i = n3 - n4;
            n6 = n3 - n4;
        }
        else {
            int n5 = n2 - n3;
            while (n4 < n5 && comparable.compareTo(array[n + n3 + n4]) >= 0) {
                n4 = (n4 << 1) + 1;
                if (n4 > 0) {
                    continue;
                }
                n4 = n5;
            }
            if (n4 > n5) {
                n4 = n5;
            }
            i = n4 + n3;
            n6 = n4 + n3;
        }
        assert -1 <= i && i < n6 && n6 <= n2;
        ++i;
        while (i < n6) {
            int n8 = i + (n6 - i >>> 1);
            if (comparable.compareTo(array[n + n8]) < 0) {
                n6 = n8;
            }
            else {
                i = n8 + 1;
            }
        }
        assert i == n6;
        return n6;
    }
    
    private void mergeLo(int n, int n2, int n3, int n4) {
        assert n2 > 0 && n4 > 0 && n + n2 == n3;
        Object[] a = this.a;
        Object[] ensureCapacity = this.ensureCapacity(n2);
        int tmpBase = this.tmpBase;
        System.arraycopy(a, n, ensureCapacity, tmpBase, n2);
        a[n++] = a[n3++];
        if (--n4 == 0) {
            System.arraycopy(ensureCapacity, tmpBase, a, n, n2);
            return;
        }
        if (n2 == 1) {
            System.arraycopy(a, n3, a, n, n4);
            a[n + n4] = ensureCapacity[tmpBase];
            return;
        }
        int minGallop = this.minGallop;
    Label_0522:
        while (true) {
            int n5 = 0;
            int n6 = 0;
            while (ComparableTimSort.$assertionsDisabled || (n2 > 1 && n4 > 0)) {
                if (((Comparable)a[n3]).compareTo(ensureCapacity[tmpBase]) < 0) {
                    a[n++] = a[n3++];
                    ++n6;
                    n5 = 0;
                    if (--n4 == 0) {
                        break Label_0522;
                    }
                }
                else {
                    a[n++] = ensureCapacity[tmpBase++];
                    ++n5;
                    n6 = 0;
                    if (--n2 == 1) {
                        break Label_0522;
                    }
                }
                if ((n5 | n6) < minGallop) {
                    continue;
                }
                while (ComparableTimSort.$assertionsDisabled || (n2 > 1 && n4 > 0)) {
                    int gallopRight = gallopRight((Comparable)a[n3], ensureCapacity, tmpBase, n2, 0);
                    if (gallopRight != 0) {
                        System.arraycopy(ensureCapacity, tmpBase, a, n, gallopRight);
                        n += gallopRight;
                        tmpBase += gallopRight;
                        n2 -= gallopRight;
                        if (n2 <= 1) {
                            break Label_0522;
                        }
                    }
                    a[n++] = a[n3++];
                    if (--n4 == 0) {
                        break Label_0522;
                    }
                    int gallopLeft = gallopLeft((Comparable)ensureCapacity[tmpBase], a, n3, n4, 0);
                    if (gallopLeft != 0) {
                        System.arraycopy(a, n3, a, n, gallopLeft);
                        n += gallopLeft;
                        n3 += gallopLeft;
                        n4 -= gallopLeft;
                        if (n4 == 0) {
                            break Label_0522;
                        }
                    }
                    a[n++] = ensureCapacity[tmpBase++];
                    if (--n2 == 1) {
                        break Label_0522;
                    }
                    --minGallop;
                    if (gallopRight >= 7 | gallopLeft >= 7) {
                        continue;
                    }
                    if (minGallop < 0) {
                        minGallop = 0;
                    }
                    minGallop += 2;
                    continue Label_0522;
                }
                throw new AssertionError();
            }
            throw new AssertionError();
        }
        this.minGallop = ((minGallop < 1) ? 1 : minGallop);
        if (n2 == 1) {
            assert n4 > 0;
            System.arraycopy(a, n3, a, n, n4);
            a[n + n4] = ensureCapacity[tmpBase];
        }
        else {
            if (n2 == 0) {
                throw new IllegalArgumentException("Comparison method violates its general contract!");
            }
            assert n4 == 0;
            assert n2 > 1;
            System.arraycopy(ensureCapacity, tmpBase, a, n, n2);
        }
        return;
    }
    
    private void mergeHi(int n, int n2, int n3, int n4) {
        assert n2 > 0 && n4 > 0 && n + n2 == n3;
        Object[] a = this.a;
        Object[] ensureCapacity = this.ensureCapacity(n4);
        int tmpBase = this.tmpBase;
        System.arraycopy(a, n3, ensureCapacity, tmpBase, n4);
        int n5 = n + n2 - 1;
        int n6 = tmpBase + n4 - 1;
        int n7 = n3 + n4 - 1;
        a[n7--] = a[n5--];
        if (--n2 == 0) {
            System.arraycopy(ensureCapacity, tmpBase, a, n7 - (n4 - 1), n4);
            return;
        }
        if (n4 == 1) {
            int n13 = n7 - n2;
            System.arraycopy(a, n5 - n2 + 1, a, n13 + 1, n2);
            a[n13] = ensureCapacity[n6];
            return;
        }
        int minGallop = this.minGallop;
    Label_0577:
        while (true) {
            int n8 = 0;
            int n9 = 0;
            while (ComparableTimSort.$assertionsDisabled || (n2 > 0 && n4 > 1)) {
                if (((Comparable)ensureCapacity[n6]).compareTo(a[n5]) < 0) {
                    a[n7--] = a[n5--];
                    ++n8;
                    n9 = 0;
                    if (--n2 == 0) {
                        break Label_0577;
                    }
                }
                else {
                    a[n7--] = ensureCapacity[n6--];
                    ++n9;
                    n8 = 0;
                    if (--n4 == 1) {
                        break Label_0577;
                    }
                }
                if ((n8 | n9) < minGallop) {
                    continue;
                }
                while (ComparableTimSort.$assertionsDisabled || (n2 > 0 && n4 > 1)) {
                    int n10 = n2 - gallopRight((Comparable)ensureCapacity[n6], a, n, n2, n2 - 1);
                    if (n10 != 0) {
                        n7 -= n10;
                        n5 -= n10;
                        n2 -= n10;
                        System.arraycopy(a, n5 + 1, a, n7 + 1, n10);
                        if (n2 == 0) {
                            break Label_0577;
                        }
                    }
                    a[n7--] = ensureCapacity[n6--];
                    if (--n4 == 1) {
                        break Label_0577;
                    }
                    int n11 = n4 - gallopLeft((Comparable)a[n5], ensureCapacity, tmpBase, n4, n4 - 1);
                    if (n11 != 0) {
                        n7 -= n11;
                        n6 -= n11;
                        n4 -= n11;
                        System.arraycopy(ensureCapacity, n6 + 1, a, n7 + 1, n11);
                        if (n4 <= 1) {
                            break Label_0577;
                        }
                    }
                    a[n7--] = a[n5--];
                    if (--n2 == 0) {
                        break Label_0577;
                    }
                    --minGallop;
                    if (n10 >= 7 | n11 >= 7) {
                        continue;
                    }
                    if (minGallop < 0) {
                        minGallop = 0;
                    }
                    minGallop += 2;
                    continue Label_0577;
                }
                throw new AssertionError();
            }
            throw new AssertionError();
        }
        this.minGallop = ((minGallop < 1) ? 1 : minGallop);
        if (n4 == 1) {
            assert n2 > 0;
            int n12 = n7 - n2;
            System.arraycopy(a, n5 - n2 + 1, a, n12 + 1, n2);
            a[n12] = ensureCapacity[n6];
        }
        else {
            if (n4 == 0) {
                throw new IllegalArgumentException("Comparison method violates its general contract!");
            }
            assert n2 == 0;
            assert n4 > 0;
            System.arraycopy(ensureCapacity, tmpBase, a, n7 - (n4 - 1), n4);
        }
        return;
    }
    
    private Object[] ensureCapacity(int min) {
        if (this.tmpLen < min) {
            int n = min | min >> 1;
            int n2 = n | n >> 2;
            int n3 = n2 | n2 >> 4;
            int n4 = n3 | n3 >> 8;
            int a = n4 | n4 >> 16;
            if (++a < 0) {
            }
            else {
                min = Math.min(a, this.a.length >>> 1);
            }
            this.tmp = new Object[min];
            this.tmpLen = min;
            this.tmpBase = 0;
        }
        return this.tmp;
    }
}
